unit lcd;

interface

uses
  Graphics,ExtCtrls,Controls,Classes;


Const
    cBG=$00d7e2d9;
    cPOff=$D2D8D3; //or $D0D6D1 or $bbc9bd
    cPOn=clBlack;
    cPSel=clRed;

type
  TLcdMatrix = array of array of boolean;
  PTImage = ^TImage;

  PTLcd = ^TLcd;

  TLcd = class (TControl)
  private
    { Private declarations }

    ImgLcd: PTImage;
    SizePixelX,
    SizePixelY,
    NbPixelX,
    NbPixelY,
    SizeImgX,
    SizeImgY:Integer;
    Matrix : TLcdMatrix;
    Link : PTLcd;

  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    Procedure SetParam(img:PTImage; spx, spy, nbx, nby:integer);

    Procedure SetLink(p:PTLcd);
    Function  GetLink:PTLCD;
    Procedure CopyLcdTo(p:PTLcd);

    Procedure Display;
    Procedure Clear;

    Function  GetPositionX(x:integer):Integer;
    Function  GetPositionY(y:integer):Integer;
    Procedure SetPixel(x,y:Integer);
    Function  GetPixel(x,y:Integer):Boolean;
    Function  InvPixel(x,y:Integer):Boolean;

    Function  DirectInvPixel(x,y:Integer):Boolean;
    Procedure DirectSetPixel(x,y:Integer;Status:Boolean);
    Function  DirectGetPixel(x,y:Integer):Boolean;

    Procedure ShiftLeft(rot:Boolean);
    Procedure ShiftRight(rot:Boolean);
    Procedure ShiftUp(rot:Boolean);
    Procedure ShiftDown(rot:Boolean);

    Procedure Inverse;
  end;

implementation

constructor TLcd.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);                                 { call inherited constructor }
  ImgLcd:=NIL;
  SizePixelX:=6;
  SizePixelY:=9;
  NbPixelX:=0;
  NbPixelY:=0;
  SizeImgX:=0;
  SizeImgY:=0;
  Link:=NIL;
end;


Procedure TLcd.SetParam(img:PTImage; spx, spy, nbx, nby:integer);
begin
  ImgLcd:=img;
  SizePixelX:=spx;
  SizePixelY:=spy;
  NbPixelX:=nbx;
  NbPixelY:=nby;
  SizeImgX:=spx*nbx+2;
  SizeImgY:=spy*nby+2;
  img.Width:=SizeImgX-1;
  img.Height:=SizeImgY-1;
  SetLength(Matrix,NbPixelX,NbPixelY);
  Display;
End;


Procedure TLcd.Display;
Var x,y:integer;
begin
  With ImgLcd.Canvas do
  Begin
    Brush.Style := bsSolid;

    //Fill the background
    Brush.Color := cBg;
    Pen.Color :=cBg;
    Rectangle(0,0,SizeImgX-1,SizeImgY-1);

    //Display off pixel
    Brush.Color := cPOff;
    Pen.Color :=cPOff;
    For x:=0 to NbPixelX-1 do
      For y:=0 to NbPixelY-1 do
        Rectangle(x*SizePixelX+1, y*SizePixelY+1, x*SizePixelX+SizePixelX,y*SizePixelY+SizePixelY);

    //Display On pixel
    Brush.Color := cPOn;
    Pen.Color :=cPOn;
    For x:=0 to NbPixelX-1 do
      For y:=0 to NbPixelY-1 do
        if Matrix[x,y] then Rectangle(x*SizePixelX+1, y*SizePixelY+1, x*SizePixelX+SizePixelX,y*SizePixelY+SizePixelY);
  End;
End;

Function TLcd.GetPositionX(x:integer):Integer;
Var p:integer;
begin
  p:=x div SizePixelX+1;
  if p>NbPixelX then p:=NbPixelX;
  GetPositionX := p;
End;

Function TLcd.GetPositionY(y:integer):Integer;
Var p:integer;
begin
  p:=y div SizePixelY+1;
  if p>NbPixelY then p:=NbPixelY;
  GetPositionY := p;
End;

Procedure TLcd.SetPixel(x,y:Integer);
var tx,ty:integer;
Begin
  tx:=GetPositionX(x)-1;
  ty:=GetPositionY(y)-1;
  Matrix[tx,ty]:=True;
End;

Procedure TLcd.DirectSetPixel(x,y:Integer;Status:Boolean);
Begin
  if (x in [0..NbPixelX-1]) and (y in [0..NbPixelY-1]) then
    Matrix[x,y]:=Status;
End;


Function TLcd.GetPixel(x,y:Integer):Boolean;
var tx,ty:integer;
Begin
  tx:=GetPositionX(x)-1;
  ty:=GetPositionY(y)-1;
  GetPixel:=Matrix[tx,ty];
End;

Function TLcd.DirectGetPixel(x,y:Integer):Boolean;
var tx,ty:integer;
Begin
  if (x in [0..NbPixelX-1]) and (y in [0..NbPixelY-1]) then
    DirectGetPixel:=Matrix[x,y];
End;

Function TLcd.InvPixel(x,y:Integer):Boolean;
var tx,ty:integer;
Begin
  tx:=GetPositionX(x)-1;
  ty:=GetPositionY(y)-1;
  Matrix[tx,ty]:=not(Matrix[tx,ty]);
  InvPixel:=Matrix[tx,ty];
  If Link<>NIL then
  Begin
    Link^.DirectInvPixel(tx,ty);
    Link^.Display;
  End;
End;

Function TLcd.DirectInvPixel(x,y:Integer):Boolean;
Begin
  Matrix[x,y]:=not(Matrix[x,y]);
  DirectInvPixel:=Matrix[x,y];
  If Link<>NIL then
  Begin
    Link^.DirectInvPixel(x,y);
    Link^.Display;
  End;
End;


Procedure TLcd.ShiftLeft(rot:Boolean);
Var x,y:integer;
    m:boolean;
Begin
  For y:=0 to NbPixelY-1 do
  Begin
    m:=Matrix[0,y];
    For x:=1 to NbPixelX-1 do
      Matrix[x-1,y]:=Matrix[x,y];
    if rot then Matrix[NbPixelX-1,y]:=m
    else Matrix[NbPixelX-1,y]:=False;
  End;
  Display;
  If Link<>NIL then Link^.ShiftLeft(rot);
End;


Procedure TLcd.ShiftRight(rot:Boolean);
Var x,y:integer;
    m:boolean;
Begin
  For y:=0 to NbPixelY-1 do
  Begin
    m:=Matrix[NbPixelX-1,y];
    For x:=NbPixelX-2 downto 0 do
      Matrix[x+1,y]:=Matrix[x,y];
    if rot then Matrix[0,y]:=m
    else Matrix[0,y]:=False;
 End;
 Display;
 If Link<>NIL then Link^.ShiftRight(rot);
End;


Procedure TLcd.ShiftUp(rot:boolean);
Var x,y:integer;
    m:boolean;
Begin
  For x:=0 to NbPixelX-1 do
  Begin
    m:=Matrix[x,0];
    For y:=1 to NbPixelY-1 do
      Matrix[x,y-1]:=Matrix[x,y];
    if rot then Matrix[x,NbPixelY-1]:=m
    else Matrix[x,NbPixelY-1]:=False;
  End;
  Display;
  If Link<>NIL then Link^.ShiftUp(rot);
End;

Procedure TLcd.ShiftDown(rot:boolean);
Var x,y:integer;
    m:Boolean;
Begin
  For x:=0 to NbPixelX-1 do
  Begin
    m:=Matrix[x,NbPixelY-1];
    For y:=NbPixelY-2 downto 0 do
      Matrix[x,y+1]:=Matrix[x,y];
    if rot then Matrix[x,0]:=m
    else Matrix[x,0]:=False;
  End;
  Display;
  If Link<>NIL then Link^.ShiftDown(rot);
End;

Procedure TLcd.Inverse;
Var x,y:integer;
Begin
  For x:=0 to NbPixelX-1 do
    For y:=0 to NbPixelY-1 do
      Matrix[x,y]:=not(Matrix[x,y]);
  Display;
  If Link<>NIL then Link^.Inverse;
End;

Procedure TLcd.SetLink(p:PTLcd);
Begin
  Link:=p;
End;

Function TLcd.GetLink:PTLCD;
Begin
  GetLink:=Link;
End;

Procedure TLcd.CopyLcdTo(p:PTLcd);
var x,y:Integer;
Begin
  If p<>NIL then
  Begin
    For x:=0 to NbPixelX-1 do
      For y:=0 to NbPixelY-1 do
        p^.DirectSetPixel(x,y,Matrix[x,y]);
    p^.Display;
  End;
End;

Procedure TLcd.Clear;
var x,y:Integer;
Begin
  For x:=0 to NbPixelX-1 do
    For y:=0 to NbPixelY-1 do
      Matrix[x,y]:=False;
  Display;
End;


end.
